const clientID = 'ENTER CLIENT ID HERE'; // Must be configured in OAuth2 provider
const redirectURI = 'http://localhost:8000/callback'; // Must be configured in OAuth2 provider
const authEndpoint = 'https://gitlab.dit.htwk-leipzig.de/oauth/authorize'; // Authorization endpoint
const tokenEndpoint = 'https://gitlab.dit.htwk-leipzig.de/oauth/token'; // Token exchange endpoint
const scopes = 'openid profile email'; // Scopes you want to request

// Utility function to generate a random string (code verifier)
function generateCodeVerifier() {
    const randomBytes = new Uint8Array(32);
    window.crypto.getRandomValues(randomBytes);
    return btoa(String.fromCharCode.apply(null, Array.from(randomBytes)))
        .replace(/\+/g, '-')
        .replace(/\//g, '_')
        .replace(/=+$/, '');
}

// Hash the code verifier to create the code challenge
async function generateCodeChallenge(verifier: string) {
    const encoder = new TextEncoder();
    const data = encoder.encode(verifier);
    const digest = await crypto.subtle.digest('SHA-256', data);
    const base64Digest = btoa(String.fromCharCode(...new Uint8Array(digest)))
        .replace(/\+/g, '-')
        .replace(/\//g, '_')
        .replace(/=+$/, '');
    return base64Digest;
}


// Build the authorization URL with the PKCE code challenge
export async function buildAuthURL(): Promise<string> {
    const codeVerifier = generateCodeVerifier();
    const codeChallenge = await generateCodeChallenge(codeVerifier);

    // Store the codeVerifier in sessionStorage for later use
    sessionStorage.setItem('pkce_code_verifier', codeVerifier);

    const params = new URLSearchParams({
        client_id: clientID,
        response_type: 'code',
        redirect_uri: redirectURI,
        scope: scopes,
        code_challenge: codeChallenge,
        code_challenge_method: 'S256',
    });

    return `${authEndpoint}?${params.toString()}`;
}

// Exchange the authorization code for an access token using the PKCE verifier
export async function exchangeCodeForToken(code: string): Promise<any> {
    // Retrieve the codeVerifier from sessionStorage
    const codeVerifier = sessionStorage.getItem('pkce_code_verifier');

    if (!codeVerifier) {
        throw new Error('No PKCE code verifier found');
    }

    const data = new URLSearchParams({
        grant_type: 'authorization_code',
        code,
        redirect_uri: redirectURI,
        client_id: clientID,
        code_verifier: codeVerifier,  // Pass the codeVerifier stored in sessionStorage
    });

    const response = await fetch(tokenEndpoint, {
        method: 'POST',
        headers: {
            'Content-Type': 'application/x-www-form-urlencoded',
        },
        body: data.toString(),
    });

    if (!response.ok) {
        throw new Error(`Failed to exchange code for token: ${response.statusText}`);
    }

    return await response.json();
}